#!/use/bin/env python
# -*- coding: utf-8 -*-

"""A simple command line tool for interacting with the arte+7 database.

arte is a great TV station and they even provide most of their programs
for 7 days after the initial airing.  But their website is truly horrible.
So this program might be useful for anyone who wants to enjoy arte
without using their website.
"""

import argparse
import asyncio
import json
import sys
from http import server
from textwrap import fill

import aiohttp
from rediscache import async_cached

try:
	import colorama
except ImportError:
	colorama = None

__version__ = '2.0.0'


def aiorun(future):
	"""Return value of a future synchronously."""
	container = []

	@asyncio.coroutine
	def wrapper():
		result = yield from future
		container.append(result)

	loop = asyncio.get_event_loop()
	loop.run_until_complete(wrapper())

	return container[0]


@async_cached('arte7', ttl=3600)
@asyncio.coroutine
def get_json(url):
	response = yield from aiohttp.get(url)
	data = yield from response.json()
	return data


@asyncio.coroutine
def get_single(url):
	"""Return description, meta and stream_url for a single video."""
	data = yield from get_json(url)
	data = data['video']

	desc = data.get('VDE')
	meta = data.get('infoProg')
	for vsr in data['VSR']:
		if vsr['VFO'] == 'HBBTV' and vsr['VQU'] == 'SQ':
			stream_url = vsr['VUR']

	return desc, meta, stream_url


def _filter(video, q):
	"""Return true if video matches the search query q."""
	def match(hay, needle):
		if hay is None:
			return False
		else:
			return hay.lower().find(needle.lower()) >= 0

	return match(video.get('TIT'), q) or match(video.get('DTW'), q)


def get_all(lang, q='', limit=1000):
	lang_map = {
		'de': 'D',
		'fr': 'F',
	}
	url = ('http://arte.tv/papi/tvguide/videos/plus7/program/%s/L3/ALL'
		'/ALL/-1/AIRDATE_DESC/%i/0/DE_FR.json') % (lang_map[lang], limit)
	data = aiorun(get_json(url))

	key = 'program%sList' % lang.upper()
	candidates = [v for v in data[key] if _filter(v, q)]
	futures = [get_single(v['VDO']['videoStreamUrl']) for v in candidates]
	responses = aiorun(asyncio.gather(*futures))

	videos = []
	for video, response in zip(candidates, responses):
		v = {
			'title': video.get('TIT'),
			'airdate': video['VDO'].get('VDA'),
			'image_url': video['IMG'].get('IUR'),
		}
		v['desc'], v['meta'], v['stream_url'] = response
		v['filename'] = v['title'] + '.' + v['stream_url'].rsplit('.', 1)[-1]
		videos.append(v)
	return videos


def format_console(videos, colors=True):
	"""Format `videos` for displaying in the console."""
	ret = []
	for video in videos:
		if colors and colorama is not None:
			tpl = (
				colorama.Fore.GREEN + "%(title)s\n" +
				colorama.Fore.YELLOW + "%(airdate)s - %(meta)s\n" +
				colorama.Fore.BLUE + "%(stream_url)s\n\n" +
				colorama.Style.RESET_ALL + fill(video["desc"] or "") + "\n\n")
		else:
			tpl = (
				"%(title)s\n" +
				"%(airdate)s - %(meta)s\n" +
				"%(stream_url)s\n\n" +
				fill(video["desc"] or "") + "\n\n")
		ret.append(tpl % video)
	return '\n'.join(ret)


def format_html(videos):
	"""Format `videos` as  HTML."""
	css = """
		.videos {
			list-style: none;
			margin: 0;
			padding: 0;
		}
		.video {
			border: 1px solid silver;
			padding: 1em;
			margin: 2em 0;
			overflow: hidden;  /* clearfix */
		}
		.video img {
			float: left;
			width: 30%;
			margin-right: 1em;
		}
		/* colors */
		a:link {
			color: #f9471b;
		}
		html {
			background-color: #202020;
			color: white;
			font-family: ProximaNova, Arial;
			padding: 1em;
		}
		body {
			background-color: transparent;
			max-width: 70em;
			margin: 0 auto;
		}
		.video {
			background-color: #ecebe9;
			color: #282828;
		}
	"""
	html_tpl = """<!DOCTYPE html>
		<html>
			<head>
				<meta charset="utf8" />
				<title>ARTE+7</title>
				<style>%(css)s</style>
			</head>
			<body>
				<h1>Arte+7</h1>
				<ul class="videos">%(list)s</ul>
			</body>
		</html>"""
	item_tpl = """<li class="video">
			<h2><a href="%(stream_url)s" download="%(filename)s">%(title)s</a></h2>
			<img src="%(image_url)s" alt="" />
			<p>%(desc)s</p>
			<span class="airdata">%(airdate)s</span>
			-
			<span class="meta">%(meta)s</span>
		</li>"""
	return html_tpl % {
		'list': '\n'.join((item_tpl % v for v in videos)),
		'css': css,
	}


class HTTPRequestHandler(server.BaseHTTPRequestHandler):
	"""HTTP Request Handler showing a single HTML page."""

	html = ''

	def do_GET(self):
		self.send_response(200)
		self.send_header("Content-type", 'text/html')
		self.end_headers()
		self.wfile.write(self.html.encode('utf8'))


def start_server(port):
	"""Start HTTP Server with HTTPRequestHandler on port `port`."""
	server_address = ('localhost', port)
	httpd = server.HTTPServer(server_address, HTTPRequestHandler)
	sa = httpd.socket.getsockname()
	print("Serving HTTP on", sa[0], "port", sa[1], "...")
	httpd.serve_forever()


def parse_args():
	"""Parse command line arguments."""
	parser = argparse.ArgumentParser(description=__doc__)
	parser.add_argument('--version', '-V', action='version', version=__version__)
	parser.add_argument('-l', '--language', choices=['de', 'fr'], default='de')
	parser.add_argument('-t', '--to',
		choices=['console', 'html', 'serve', 'json'],
		default='console', help="output format")
	parser.add_argument('-b', '--boring', action='store_true',
		help="no colors in console output")
	parser.add_argument('-p', '--port', type=int, help="port used for serve",
		default=8000)
	parser.add_argument('q', nargs='?', help="search term", default='')
	return parser.parse_args()


def main():
	"""Entry point for the command line tool."""
	args = parse_args()
	videos = get_all(args.language, q=args.q)

	if args.to == 'html':
		print(format_html(videos))
	elif args.to == 'serve':
		HTTPRequestHandler.html = format_html(videos)
		start_server(args.port)
	elif args.to == 'json':
		print(json.dumps(videos, indent=2))
	else:
		colors = not args.boring and sys.stdout.isatty()
		print(format_console(videos, colors=colors))


if __name__ == '__main__':
	main()
